window.Gleam.initSearch([{"doc":"toy","title":"toy","content":"# Toy\n\nPractical, composable dynamic decoders and validators for Gleam!\n\n[![Package Version](https://img.shields.io/hexpm/v/toy)](https://hex.pm/packages/toy)\n[![Hex Docs](https://img.shields.io/badge/hex-docs-ffaff3)](https://hexdocs.pm/toy/)\n\n```sh\ngleam add toy\n```\n```gleam\nimport gleam/dynamic\nimport gleam/json\nimport toy\n\npub type User {\n  User(name: String, height: Float, age: Int)\n}\n\npub fn user_decoder() {\n  use name &lt;- toy.field(&quot;name&quot;, toy.string)\n  use height &lt;- toy.field(&quot;height&quot;, toy.float |&gt; toy.float_min(0.0))\n  use age &lt;- toy.field(&quot;age&quot;, toy.int |&gt; toy.int_min(0))\n\n  toy.decoded(User(name:, height:, age:))\n}\n\npub fn main() {\n  let assert Ok(data) =\n    json.decode(\n      &quot;\n  {\n    \\&quot;name\\&quot;: \\&quot;Alice\\&quot;,\n    \\&quot;height\\&quot;: 1.7,\n    \\&quot;age\\&quot;: 25\n  }\n  &quot;,\n      dynamic.dynamic,\n    )\n\n  let user = data |&gt; toy.decode(user_decoder())\n}\n```\n\n## Goals\n\nToy aims to provide a simple to use, composable api for decoding and validating\ndynamic data in Gleam. It is inspired by the gleam standard library,\n[decode](https://hexdocs.pm/decode/) and [Zod](https://zod.dev/).\n\nToy strives to satisfy all of the following goals:\n- Composable\n- Good DX\n- Type safe\n- No opaque types\n- Minimize surface for dumb mistakes\n- Return all error messages possible\n- Allow for easy internationalization of error messages\n- Provide validation functions for common use cases\n\n## Why not existing solutions?\n\nIn application development you most of the time decode dynamic values\ninto records (objects in other lanugages).\nIt might be a JSON payload from an API, a configuration file,\nor a database row. So this will be the main focus of this comparison.\n\n### Gleam standard library [gleam_stdlib/dynamic](https://hexdocs.pm/gleam_stdlib/gleam/dynamic.html)\n\n```gleam\npub fn user_decoder_stdlib() {\n  dynamic.decode3(\n    User,\n    dynamic.field(&quot;name&quot;, dynamic.string),\n    dynamic.field(&quot;height&quot;, dynamic.float),\n    dynamic.field(&quot;age&quot;, dynamic.int),\n  )\n}\n```\n\nThe standard library requires two thing from the developer. You have to\nuse the right function, `decode3` in this case, because the user has 3 fields\nand *you have to list the fields in the same order as the record definition*.\n\nThe first point is generally not a problem until you reach 10 fields, because\nthere is no `decode10` function. The second point is a bigger problem.\nWhen developing, it is very common to modify existing records (add/remove fields).\nWhen you do this, it is very easy to mess up the order int the decoder. And if the\ntwo fields you swapped are both strings, the compiler will not catch this mistake.\n**You will end up with bad data in your application**.\n\n### The decode package [decode](https://hexdocs.pm/decode/)\n\n```gleam\npub fn user_decoder_decode() {\n  decode.into({\n    use name &lt;- decode.parameter\n    use height &lt;- decode.parameter\n    use age &lt;- decode.parameter\n    User(name, height, age)\n  })\n  |&gt; decode.field(&quot;name&quot;, decode.string)\n  |&gt; decode.field(&quot;height&quot;, decode.float)\n  |&gt; decode.field(&quot;age&quot;, decode.int)\n}\n```\n\nThe decode package is a bit better than the standard library. Now you can decode\nrecords with any number of fields in the same way easily. However the second\nproblem still persists. The second `decode.field(&quot;height&quot;, decode.float)` call \nhas to match up with the second `use height &lt;- decode.parameter` call, which\nneeds to match up with the right field in the record definition. If you mess\nthis up during refactoring or developing a new feature, **you will end up with\nbad data in your application.**\n\n### Toy\n\n```gleam\npub fn user_decoder_toy() {\n  use name &lt;- toy.field(&quot;name&quot;, toy.string)\n  use height &lt;- toy.field(&quot;height&quot;, toy.float)\n  use age &lt;- toy.field(&quot;age&quot;, toy.int)\n\n  toy.decoded(User(name:, height:, age:))\n}\n```\n\nToy solves both problems. Each field is defined on a single line. And since\ngleam 1.4.0, we can construct the record with the *label shorthand* syntax.\nBy using this syntax the compiler guarantees that the *name* value will be\nassigned to the name field in the record, regardless of the order you specify\nthem in. This makes is much harder to mess up the decoder when refactoring.\n\nBut there is a catch. **Every line of the decoder will be executed on every\ninvocation**. To return errors for all fields at once, we need to continue\ndecoding even after the first error. *But what will be in the `name` variable\nif the `name` field failed to decode?* The answer is a default string value\n(empty string in this case). So you have to make sure to not perform any\nside effects in the decoder, because withing the decoder function you are never\nguaranteed that the data is actually valid.\n\n```gleam\npub fn user_decoder_toy() {\n  use name &lt;- toy.field(&quot;name&quot;, toy.string)\n  // This is bad, you should never perform side effects in the decoder\n  io.println(name)\n  use height &lt;- toy.field(&quot;height&quot;, toy.float)\n  use age &lt;- toy.field(&quot;age&quot;, toy.int)\n\n  toy.decoded(User(name:, height:, age:))\n}\n```\n\nThis is not as scary as it seems. When you are using toy and decode a value with\nit, you will always get valid data. The rule only applies withing the decoder\nitself.\n\nAnd in practice, the rule *don&#39;t perform side effects in the decoder* is much\neasier to follow than the rule *make sure the field order in the decoder always\nmatches the record definition*.\n\n## Usage\n\nHere is a complex example of a decoder, including validation. This should give\nyou a good overview of the capabilities of toy.\n\n```gleam\nimport birl\nimport gleam/dynamic\nimport gleam/io\nimport gleam/json\nimport gleam/option.{type Option}\nimport gleam/order\nimport gleam/result\nimport toy\n\npub type Guest {\n  Guest(name: String, age: Int, special_needs: Option(String))\n}\n\npub type Table {\n  Any\n  Specific(table_number: Int)\n}\n\npub type Reservation {\n  Reservation(\n    person_name: String,\n    email: String,\n    datetime: birl.Time,\n    guests: List(Guest),\n    table: Table,\n    notes: Option(String),\n  )\n}\n\npub fn time_decoder() {\n  toy.string\n  |&gt; toy.try_map(birl.now(), fn(val) {\n    birl.parse(val)\n    |&gt; result.replace_error([toy.ToyError(toy.InvalidType(&quot;DateTime&quot;, val), [])])\n  })\n}\n\npub fn time_future(val: birl.Time) {\n  case birl.compare(val, birl.now()) {\n    order.Gt -&gt; Ok(Nil)\n    _ -&gt;\n      Error([\n        toy.ToyError(\n          toy.ValidationFailed(&quot;date_future&quot;, &quot;DateTime&quot;, birl.to_http(val)),\n          [],\n        ),\n      ])\n  }\n}\n\npub fn reservation_decoder() {\n  use person_name &lt;- toy.field(&quot;person_name&quot;, toy.string |&gt; toy.string_nonempty)\n  use email &lt;- toy.field(&quot;email&quot;, toy.string |&gt; toy.string_email)\n  use datetime &lt;- toy.field(\n    &quot;datetime&quot;,\n    time_decoder() |&gt; toy.refine(time_future),\n  )\n  use guests &lt;- toy.field(\n    &quot;guests&quot;,\n    toy.list({\n      use name &lt;- toy.field(&quot;name&quot;, toy.string)\n      use age &lt;- toy.field(&quot;age&quot;, toy.int |&gt; toy.int_min(18))\n      use special_needs &lt;- toy.optional_field(&quot;special_needs&quot;, toy.string)\n      toy.decoded(Guest(name:, age:, special_needs:))\n    })\n      |&gt; toy.list_nonempty,\n  )\n  use table &lt;- toy.field(&quot;table&quot;, {\n    use type_ &lt;- toy.field(&quot;type&quot;, toy.string)\n\n    case type_ {\n      &quot;specific&quot; -&gt; {\n        use table_number &lt;- toy.field(&quot;table_number&quot;, toy.int)\n        toy.decoded(Specific(table_number:))\n      }\n      &quot;any&quot; -&gt; toy.decoded(Any)\n      _ -&gt; toy.fail(toy.InvalidType(&quot;Table&quot;, type_), Any)\n    }\n  })\n  use notes &lt;- toy.field(&quot;notes&quot;, toy.string |&gt; toy.nullable)\n  toy.decoded(Reservation(\n    person_name:,\n    email:,\n    datetime:,\n    guests:,\n    table:,\n    notes:,\n  ))\n}\n\nconst data = &quot;\n{\n  \\&quot;person_name\\&quot;: \\&quot;Alice\\&quot;,\n  \\&quot;email\\&quot;: \\&quot;alice@example.com\\&quot;,\n  \\&quot;datetime\\&quot;: \\&quot;2025-01-01T12:00:00Z\\&quot;,\n  \\&quot;guests\\&quot;: [\n    { \\&quot;name\\&quot;: \\&quot;Alice\\&quot;, \\&quot;age\\&quot;: 25 },\n    { \\&quot;name\\&quot;: \\&quot;Bob\\&quot;, \\&quot;age\\&quot;: 30, \\&quot;special_needs\\&quot;: \\&quot;Vegan\\&quot; }\n  ],\n  \\&quot;table\\&quot;: { \\&quot;type\\&quot;: \\&quot;specific\\&quot;, \\&quot;table_number\\&quot;: 1 },\n  \\&quot;notes\\&quot;: null\n}\n&quot;\n\npub fn main() {\n  let assert Ok(data) = json.decode(data, dynamic.dynamic)\n\n  data |&gt; toy.decode(reservation_decoder()) |&gt; io.debug\n}\n```\n\nFurther documentation can be found at &lt;https://hexdocs.pm/toy&gt;.\n\n","url":"index.html"},{"doc":"toy","title":"Decoder","content":"pub type Decoder(a) =\n  fn(dynamic.Dynamic) -&gt; #(a, Result(a, List(ToyError)))\n A decoder is a function that takes a `Dynamic` value and returns a tuple\n containing the default value of the same type and a `Result` with the decoded\n value or a list of errors\n\nSynonyms:\ntoy.Decoder\ntoy Decoder","url":"toy.html#Decoder"},{"doc":"toy","title":"ToyError","content":"pub type ToyError {\n  ToyError(error: ToyFieldError, path: List(String))\n}\n Contains decoding or validation errors\nToyError(error: ToyFieldError, path: List(String))\n\n\nSynonyms:\ntoy.ToyError\ntoy ToyError","url":"toy.html#ToyError"},{"doc":"toy","title":"ToyFieldError","content":"pub type ToyFieldError {\n  InvalidType(expected: String, found: String)\n  Missing(expected: String)\n  ValidationFailed(\n    check: String,\n    expected: String,\n    found: String,\n  )\n}\n Each type of error that can be returned by the decoders\nInvalidType(expected: String, found: String)\n\n\nMissing(expected: String)\n\n\nValidationFailed(check: String, expected: String, found: String)\n\n\nSynonyms:\ntoy.ToyFieldError\ntoy ToyFieldError","url":"toy.html#ToyFieldError"},{"doc":"toy","title":"bit_array","content":"pub fn bit_array(\n  data: Dynamic,\n) -&gt; #(BitArray, Result(BitArray, List(ToyError)))\n Decode a `BitArray`\nSynonyms:\ntoy.bit_array\ntoy bit_array","url":"toy.html#bit_array"},{"doc":"toy","title":"decode","content":"pub fn decode(\n  data: Dynamic,\n  decoder: fn(Dynamic) -&gt; #(a, Result(a, List(ToyError))),\n) -&gt; Result(a, List(ToyError))\n Takes a `Dynamic` value and runs a `Decoder` on it, returning the result\n of the decoding process\nSynonyms:\ntoy.decode\ntoy decode","url":"toy.html#decode"},{"doc":"toy","title":"decoded","content":"pub fn decoded(\n  value: a,\n) -&gt; fn(Dynamic) -&gt; #(a, Result(a, List(ToyError)))\n Creates a decoder which directly returns the provided value.\n It is useful when decoding records.\n\n \n pub fn user_decoder() {\n   use name &lt;- toy.field(&quot;name&#39;, toy.string)\n   toy.decoded(User(:name))\n }\n \nSynonyms:\ntoy.decoded\ntoy decoded","url":"toy.html#decoded"},{"doc":"toy","title":"dynamic","content":"pub fn dynamic(data: a) -&gt; #(Dynamic, Result(a, b))\n Always decodes the provided value as `Dynamic`.\n Error is never returned from this decoder\nSynonyms:\ntoy.dynamic\ntoy dynamic","url":"toy.html#dynamic"},{"doc":"toy","title":"fail","content":"pub fn fail(\n  error: ToyFieldError,\n  default: a,\n) -&gt; fn(Dynamic) -&gt; #(a, Result(a, List(ToyError)))\n Returns a decoder that always fails with the provided error\nSynonyms:\ntoy.fail\ntoy fail","url":"toy.html#fail"},{"doc":"toy","title":"field","content":"pub fn field(\n  key: a,\n  decoder: fn(Dynamic) -&gt; #(b, Result(b, List(ToyError))),\n  next: fn(b) -&gt; fn(Dynamic) -&gt; #(c, Result(c, List(ToyError))),\n) -&gt; fn(Dynamic) -&gt; #(c, Result(c, List(ToyError)))\n Decode a field from a `Dynamic` value\n\n This function will index into dictionary with any key type, tuples with\n integer or javascript arrays and objects. The value found under the key\n will be decoded with the provided decoder.\n\n \n pub fn user_decoder() {\n   use name &lt;- toy.field(&quot;name&#39;, toy.string)\n   toy.decoded(User(:name))\n }\n \nSynonyms:\ntoy.field\ntoy field","url":"toy.html#field"},{"doc":"toy","title":"float","content":"pub fn float(\n  data: Dynamic,\n) -&gt; #(Float, Result(Float, List(ToyError)))\n Decode a `Float` value\nSynonyms:\ntoy.float\ntoy float","url":"toy.html#float"},{"doc":"toy","title":"float_max","content":"pub fn float_max(\n  dec: fn(Dynamic) -&gt; #(Float, Result(Float, List(ToyError))),\n  maximum: Float,\n) -&gt; fn(Dynamic) -&gt; #(Float, Result(Float, List(ToyError)))\n Validates that number is less than the provided maximum\n\n **Error type**: `float_max`\nSynonyms:\ntoy.float_max\ntoy float_max","url":"toy.html#float_max"},{"doc":"toy","title":"float_min","content":"pub fn float_min(\n  dec: fn(Dynamic) -&gt; #(Float, Result(Float, List(ToyError))),\n  minimum: Float,\n) -&gt; fn(Dynamic) -&gt; #(Float, Result(Float, List(ToyError)))\n Validates that number is greater or equal to the provided minimum\n\n **Error type**: `float_min`\nSynonyms:\ntoy.float_min\ntoy float_min","url":"toy.html#float_min"},{"doc":"toy","title":"float_range","content":"pub fn float_range(\n  dec: fn(Dynamic) -&gt; #(Float, Result(Float, List(ToyError))),\n  minimum: Float,\n  maximum: Float,\n) -&gt; fn(Dynamic) -&gt; #(Float, Result(Float, List(ToyError)))\n Validates that the number is withing the provided range [minimum, maximum)\n\n **Error type**: `float_range`\nSynonyms:\ntoy.float_range\ntoy float_range","url":"toy.html#float_range"},{"doc":"toy","title":"int","content":"pub fn int(data: Dynamic) -&gt; #(Int, Result(Int, List(ToyError)))\n Decode an `Int` value\nSynonyms:\ntoy.int\ntoy int","url":"toy.html#int"},{"doc":"toy","title":"int_max","content":"pub fn int_max(\n  dec: fn(Dynamic) -&gt; #(Int, Result(Int, List(ToyError))),\n  maximum: Int,\n) -&gt; fn(Dynamic) -&gt; #(Int, Result(Int, List(ToyError)))\n Validates that number is less than the provided maximum\n\n **Error type**: `int_max`\nSynonyms:\ntoy.int_max\ntoy int_max","url":"toy.html#int_max"},{"doc":"toy","title":"int_min","content":"pub fn int_min(\n  dec: fn(Dynamic) -&gt; #(Int, Result(Int, List(ToyError))),\n  minimum: Int,\n) -&gt; fn(Dynamic) -&gt; #(Int, Result(Int, List(ToyError)))\n Validates that number is greater or equal to the provided minimum\n\n **Error type**: `int_min`\nSynonyms:\ntoy.int_min\ntoy int_min","url":"toy.html#int_min"},{"doc":"toy","title":"int_range","content":"pub fn int_range(\n  dec: fn(Dynamic) -&gt; #(Int, Result(Int, List(ToyError))),\n  minimum: Int,\n  maximum: Int,\n) -&gt; fn(Dynamic) -&gt; #(Int, Result(Int, List(ToyError)))\n Validates that number is in the provided range: [minimum, maximum)\n\n **Error type**: `int_range`\nSynonyms:\ntoy.int_range\ntoy int_range","url":"toy.html#int_range"},{"doc":"toy","title":"list","content":"pub fn list(\n  item: fn(Dynamic) -&gt; #(a, Result(a, List(ToyError))),\n) -&gt; fn(Dynamic) -&gt; #(List(a), Result(List(a), List(ToyError)))\n Decode a list of values\n\n \n pub fn fruits_decoder() {\n   toy.list({\n     use name &lt;- toy.field(&quot;name&quot;, toy.string)\n     toy.decoded(Fruit(:name))\n   })\n }\nSynonyms:\ntoy.list\ntoy list","url":"toy.html#list"},{"doc":"toy","title":"list_max","content":"pub fn list_max(\n  dec: fn(Dynamic) -&gt; #(List(a), Result(List(a), List(ToyError))),\n  maximum: Int,\n) -&gt; fn(Dynamic) -&gt; #(List(a), Result(List(a), List(ToyError)))\n Validates that the length of the list is less than the provided maximum\n\n **Error type**: `list_max`\nSynonyms:\ntoy.list_max\ntoy list_max","url":"toy.html#list_max"},{"doc":"toy","title":"list_min","content":"pub fn list_min(\n  dec: fn(Dynamic) -&gt; #(List(a), Result(List(a), List(ToyError))),\n  minimum: Int,\n) -&gt; fn(Dynamic) -&gt; #(List(a), Result(List(a), List(ToyError)))\n Validates that the length of the list is at least the provided number\n\n **Error type**: `list_min`\nSynonyms:\ntoy.list_min\ntoy list_min","url":"toy.html#list_min"},{"doc":"toy","title":"list_nonempty","content":"pub fn list_nonempty(\n  dec: fn(Dynamic) -&gt; #(List(a), Result(List(a), List(ToyError))),\n) -&gt; fn(Dynamic) -&gt; #(List(a), Result(List(a), List(ToyError)))\n Validates that the list is not empty (contains at least one element)\n\n **Error type**: `list_nonempty`\nSynonyms:\ntoy.list_nonempty\ntoy list_nonempty","url":"toy.html#list_nonempty"},{"doc":"toy","title":"map","content":"pub fn map(\n  dec: fn(Dynamic) -&gt; #(a, Result(a, List(ToyError))),\n  fun: fn(a) -&gt; b,\n) -&gt; fn(Dynamic) -&gt; #(b, Result(b, List(ToyError)))\n Map the result of the decoder to a new value\n\n \n pub type Unit {\n   Centimeters(Int)\n   Milimeters(Int)\n }\n\n pub type User {\n   User(height: Unit)\n }\n\n pub fn user_decoder() {\n   use height &lt;- toy.field(&quot;height&quot;, toy.int |&gt; toy.map(Centimeters))\n   toy.decoded(User(:height))\n }\nSynonyms:\ntoy.map\ntoy map","url":"toy.html#map"},{"doc":"toy","title":"nullable","content":"pub fn nullable(\n  of dec: fn(Dynamic) -&gt; #(a, Result(a, List(ToyError))),\n) -&gt; fn(Dynamic) -&gt;\n  #(Option(a), Result(Option(a), List(ToyError)))\n Creates a new decoder from an existing one, which will return `None` if\n the value is `null` or `undefined` on javascript, or `nil`, `null`,\n `undefined` on erlang. Otherwise it will return the result of the provided\n decoder wrapped in `Some`\nSynonyms:\ntoy.nullable\ntoy nullable","url":"toy.html#nullable"},{"doc":"toy","title":"option","content":"pub fn option(\n  of dec: fn(Dynamic) -&gt; #(a, Result(a, List(ToyError))),\n) -&gt; fn(Dynamic) -&gt;\n  #(Option(a), Result(Option(a), List(ToyError)))\n Decodes a gleam `Option` type. In erlang represented as `{ok, Value}` or `none`.\n In javascript represented as an instance of `Some` or `None` classes.\nSynonyms:\ntoy.option\ntoy option","url":"toy.html#option"},{"doc":"toy","title":"optional_field","content":"pub fn optional_field(\n  key: a,\n  decoder: fn(Dynamic) -&gt; #(b, Result(b, List(ToyError))),\n  next: fn(Option(b)) -&gt;\n    fn(Dynamic) -&gt; #(c, Result(c, List(ToyError))),\n) -&gt; fn(Dynamic) -&gt; #(c, Result(c, List(ToyError)))\n Decode a field from a `Dynamic` value\n\n This function will index into dictionary with any key type, tuples with\n integer or javascript arrays and objects. The value found under the key\n will be decoded with the provided decoder.\n\n `None` is returned only if the field is missing. Otherwise the provided\n decoder is used to decode the value.\n\n \n pub fn reservation_decoder() {\n   use note &lt;- toy.optional_field(&quot;note&#39;, toy.string)\n   toy.decoded(User(:name))\n }\n \nSynonyms:\ntoy.optional_field\ntoy optional_field","url":"toy.html#optional_field"},{"doc":"toy","title":"refine","content":"pub fn refine(\n  dec: fn(Dynamic) -&gt; #(a, Result(a, List(ToyError))),\n  fun: fn(a) -&gt; Result(Nil, List(ToyError)),\n) -&gt; fn(Dynamic) -&gt; #(a, Result(a, List(ToyError)))\n Refine the result of the decoder with a validation function\n\n \n pub fn user_decoder() {\n   use name &lt;- toy.field(&quot;name&quot;, toy.string |&gt; toy.refine(fn(name) {\n     case name {\n       &quot;toy&quot; -&gt; Error([toy.ToyError(toy.ValidationFailed(&quot;name_taken&quot;, &quot;new_name&quot;, name), [])])\n       _ -&gt; Ok(Nil)\n     }\n   }))\n  toy.decoded(User(:name))\n }\nSynonyms:\ntoy.refine\ntoy refine","url":"toy.html#refine"},{"doc":"toy","title":"string","content":"pub fn string(\n  data: Dynamic,\n) -&gt; #(String, Result(String, List(ToyError)))\n Decode a `String` value\nSynonyms:\ntoy.string\ntoy string","url":"toy.html#string"},{"doc":"toy","title":"string_email","content":"pub fn string_email(\n  dec: fn(Dynamic) -&gt; #(String, Result(String, List(ToyError))),\n) -&gt; fn(Dynamic) -&gt; #(String, Result(String, List(ToyError)))\n Validates that the string contains an email address.\n This is done by checking if the string contains the &quot;@&quot; character.\n\n **Error type**: `string_email`\nSynonyms:\ntoy.string_email\ntoy string_email","url":"toy.html#string_email"},{"doc":"toy","title":"string_max","content":"pub fn string_max(\n  dec: fn(Dynamic) -&gt; #(String, Result(String, List(ToyError))),\n  maximum: Int,\n) -&gt; fn(Dynamic) -&gt; #(String, Result(String, List(ToyError)))\n Validates that the length of the string is less than the provided number\n\n **Error type**: `string_max`\nSynonyms:\ntoy.string_max\ntoy string_max","url":"toy.html#string_max"},{"doc":"toy","title":"string_min","content":"pub fn string_min(\n  dec: fn(Dynamic) -&gt; #(String, Result(String, List(ToyError))),\n  minimum: Int,\n) -&gt; fn(Dynamic) -&gt; #(String, Result(String, List(ToyError)))\n Validates that the length of the string is at least the provided number\n\n **Error type**: `string_min`\nSynonyms:\ntoy.string_min\ntoy string_min","url":"toy.html#string_min"},{"doc":"toy","title":"string_nonempty","content":"pub fn string_nonempty(\n  dec: fn(Dynamic) -&gt; #(String, Result(String, List(ToyError))),\n) -&gt; fn(Dynamic) -&gt; #(String, Result(String, List(ToyError)))\n Validates that the string contains some characters that are not whitespace.\n\n **Error type**: `string_nonempty`\nSynonyms:\ntoy.string_nonempty\ntoy string_nonempty","url":"toy.html#string_nonempty"},{"doc":"toy","title":"try_map","content":"pub fn try_map(\n  dec: fn(Dynamic) -&gt; #(a, Result(a, List(ToyError))),\n  default: b,\n  fun: fn(a) -&gt; Result(b, List(ToyError)),\n) -&gt; fn(Dynamic) -&gt; #(b, Result(b, List(ToyError)))\n Map the result of the decoder to a new value or return an error\n\n \n pub fn user_decoder() {\n   use name &lt;- toy.field(&quot;name&quot;, toy.string |&gt; toy.try_map(&quot;&quot;, fn(name) {\n     case name {\n       &quot;toy&quot; -&gt; Error([toy.ToyError(toy.ValidationFailed(&quot;name_taken&quot;, &quot;new_name&quot;, name), [])])\n       _ -&gt; Ok(string.uppercase(name))\n     }\n   }))\n  toy.decoded(User(:name))\n }\nSynonyms:\ntoy.try_map\ntoy try_map","url":"toy.html#try_map"},{"doc":"toy","title":"toy","content":"","url":"toy.html"}]);